@@ -1,26 +1,25 @@
 import pluginVue from "eslint-plugin-vue";
-import vueTsEslintConfig from "@vue/eslint-config-typescript";
+import {
+  defineConfigWithVueTs,
+  vueTsConfigs,
+} from "@vue/eslint-config-typescript";
 import prettierConfig from "@vue/eslint-config-prettier";
 
-export default [
+export default defineConfigWithVueTs(
   {
     name: "app/files-to-lint",
     files: ["**/*.{ts,mts,tsx,vue}"],
   },
-
   {
     name: "app/files-to-ignore",
     ignores: ["**/dist/**", "**/dist-ssr/**", "**/coverage/**"],
   },
-
-  ...pluginVue.configs["flat/essential"],
-  ...vueTsEslintConfig(),
+  pluginVue.configs["flat/essential"],
+  vueTsConfigs.recommended,
   prettierConfig,
-
   {
     rules: {
       // Note: you must disable the base rule as it can report incorrect errors
-      "no-unused-expressions": "off",
       "@typescript-eslint/no-unused-expressions": "off",
       // TODO: theres too many of these from before ts
       "@typescript-eslint/no-explicit-any": "off",
@@ -34,5 +33,5 @@ export default [
         },
       ],
     },
-  },
-];
+  }
+);
